/**
  *  \file t_iterator.cc
  *
  *  (c) 2001-2009 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "cpluslib/iterator.h"

#include "tests.h"
#include <vector>
#include <map>

void
TestIterator::testPointerIterator()
{
    std::vector<int*> pvec;
    pvec.push_back(new int(1));
    pvec.push_back(new int(2));
    pvec.push_back(new int(3));
    pvec.push_back(new int(17));
    pvec.push_back(new int(42));
    pvec.push_back(new int(-1));

    typedef dereferencing_iterator<std::vector<int*>::iterator> ite;

    ite i = ite(pvec.begin());
    TS_ASSERT_EQUALS(*i, 1);  TS_ASSERT_DIFFERS(i, ite(pvec.end())); ++i;
    TS_ASSERT_EQUALS(*i, 2);  TS_ASSERT_DIFFERS(i, ite(pvec.end())); ++i;
    TS_ASSERT_EQUALS(*i, 3);  TS_ASSERT_DIFFERS(i, ite(pvec.end())); ++i;
    TS_ASSERT_EQUALS(*i, 17); TS_ASSERT_DIFFERS(i, ite(pvec.end())); ++i;
    TS_ASSERT_EQUALS(*i, 42); TS_ASSERT_DIFFERS(i, ite(pvec.end())); ++i;
    TS_ASSERT_EQUALS(*i, -1); TS_ASSERT_DIFFERS(i, ite(pvec.end())); ++i;
    TS_ASSERT_EQUALS(i, ite(pvec.end()));
}

void
TestIterator::testMemberIterator()
{
    std::map<int,int*> imap;
    imap[1] = new int(17);
    imap[17] = new int(42);
    imap[42] = new int(3);

    // Member iterator
    {
        typedef member_iterator<std::map<int,int*>::iterator,
                                const int,
                                &std::pair<const int,int*>::first> mite;

        mite i = mite(imap.begin());
        TS_ASSERT_EQUALS(*i, 1);  TS_ASSERT_DIFFERS(i, mite(imap.end())); ++i;
        TS_ASSERT_EQUALS(*i, 17); TS_ASSERT_DIFFERS(i, mite(imap.end())); ++i;
        TS_ASSERT_EQUALS(*i, 42); TS_ASSERT_DIFFERS(i, mite(imap.end())); ++i;
        TS_ASSERT_EQUALS(i, mite(imap.end()));
    }

    // Member + Pointer iterator
    {
        typedef dereferencing_iterator<member_iterator<std::map<int,int*>::iterator,
                                                       int*,
                                                       &std::pair<const int,int*>::second> > dite;
        dite i = dite(imap.begin());
        TS_ASSERT_EQUALS(*i, 17); TS_ASSERT_DIFFERS(i, dite(imap.end())); ++i;
        TS_ASSERT_EQUALS(*i, 42); TS_ASSERT_DIFFERS(i, dite(imap.end())); ++i;
        TS_ASSERT_EQUALS(*i, 3);  TS_ASSERT_DIFFERS(i, dite(imap.end())); ++i;
        TS_ASSERT_EQUALS(i, dite(imap.end()));
    }
}
