/**
  *  \file t_smallset.cc
  *
  *  (c) 2001-2009 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "cpluslib/smallset.h"

#include "tests.h"

enum Numbers { One, Two, Three };

typedef SmallSet<Numbers> Num_set;

void
TestSmallSet::testIt()
{
    Num_set e;
    Num_set s1(One), s2(Two), s3(Three);

    TS_ASSERT(e.empty());
    TS_ASSERT(!s1.empty());
    TS_ASSERT(!s2.empty());
    TS_ASSERT(!s3.empty());

    TS_ASSERT(s1.contains(One));
    TS_ASSERT(!s1.contains(Two));
    TS_ASSERT(!s1.contains(Three));

    TS_ASSERT((s1 + s2).contains(One));
    TS_ASSERT(!(s1 + s2).contains(Three));
    TS_ASSERT((s1 + s2).contains(s1));
    TS_ASSERT((s1 + s2).contains(s2));
    TS_ASSERT(!(s1 + s2).contains(s3));

    TS_ASSERT_DIFFERS(s1, s2);
    TS_ASSERT_EQUALS(s1 + Two - One, s2);

    Num_set n(s1);
    n += Two;
    TS_ASSERT(n.containsAnyOf(s1));
    TS_ASSERT(n.contains(s1));
    TS_ASSERT(!s1.contains(n));

    TS_ASSERT_EQUALS((n ^ s1), s2);
    TS_ASSERT_EQUALS((s1 ^ s2), n);
    TS_ASSERT_EQUALS((n ^ s2), s1);

    TS_ASSERT((s1 & s2).empty());
    TS_ASSERT(!(s1 - s2).empty());

    TS_ASSERT_EQUALS((n - s2), s1);
}
